/* 
 * SDLInvasion a Space Invader clone.
 * Copyright (C) 2002 Laurent GOMEZ & MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */


#define _PLAYER_C

#include <stdio.h>
#include <stdlib.h>

#include <assert.h> /* for debug */

#include "SDL_image.h"

#include "player.h"


#define PLAYER_Y_OFFSET 55 /* offset to apply to Player sprite */


/* 
 * Create a new player.
 */
Player *Player_new(SDL_Surface *screen, char *player_sprite) {
   Player *player = NULL;
   View_Player *view_player = NULL; 
   SDL_Surface *image = NULL;

   /* Alloc Player */
   player = malloc(sizeof(Player));
   if(!player) { fprintf(stderr, "Can't allocate memory (Player)\n"); exit(0); }

   player->alive = 3; /* player start with 3 life */

   /* Alloc View_Player */
   view_player = malloc(sizeof(View_Player));
   if(!view_player) { fprintf(stderr, "Can't allocate memory (View_Player)\n"); exit(0); }
   
   /* Load image */
   image = IMG_Load(player_sprite);
   if(image == NULL) {
     fprintf(stderr, "Couldn't load %s: %s\n", player_sprite, SDL_GetError());
     return;
   }
   /* + gestion transparence... */
   view_player->sprite = SDL_DisplayFormat(image);
   SDL_FreeSurface(image);

   /* Init position and velocity to 0 */
   view_player->x = screen->w / 2;;
   view_player->y = screen->h - view_player->sprite->h - PLAYER_Y_OFFSET;
   view_player->xvel = 0;

   /* Connect a view with the Player */
   player->view = view_player;
   
   return player;
}

/* 
 * Destroy a player.
 */
int Player_destroy(Player *player) {
   /* Assertion */
   assert(player && player->view);

   /* Free memory */
   free(player->view);
   free(player);

   return 1;
}

/*
 * Player is alive ? return nb of life.
 */
int Player_isAlive(Player *player) {
   /* Assertion */
   assert(player);

   return player->alive;
}

/*
 * Player velocity.
 */
int Player_xvel(Player *player) {
   /* Assertion */
   assert(player && player->view);

   return player->view->xvel;
}

/* 
 * Move the player on the left.
 */
int Player_moveLeft(Player *player) {
   /* Assertion */
   assert(player && player->view);

   player->view->xvel = -1;

   return 1;
}

/* 
 * Move the player on the right.
 */
int Player_moveRight(Player *player) {
   /* Assertion */
   assert(player && player->view);

   player->view->xvel = +1;

   return 1;
}

/* 
 * Stop the player.
 */
int Player_stop(Player *player) {
   /* Assertion */
   assert(player && player->view);

   player->view->xvel = 0;

   return 1;
}


/* 
 * Display the player on screen.
 */
int Player_draw(Player *player, SDL_Surface *screen) {
   SDL_Rect dstrect;

   /* Assertion */
   assert(player && player->view);  
   assert(screen);

   /* Update player position */
   player->view->x += player->view->xvel;
   if(player->view->x < 0) {
      player->view->x = 0; 
      player->view->xvel = 0;
   }
   else if(player->view->x > screen->w - player->view->sprite->w - 1) {
      player->view->x = screen->w - player->view->sprite->w - 1;
      player->view->xvel = 0;
   }
   /*fprintf(stdout, "player pos(%d,%d)\n", player->view->x, player->view->y);*/

   /* Erase with black */
   dstrect.x = 0;
   dstrect.y = screen->h - player->view->sprite->h - PLAYER_Y_OFFSET;
   dstrect.w = screen->w;
   dstrect.h = player->view->sprite->h;
   SDL_FillRect(screen, &dstrect, SDL_MapRGB(screen->format, 0, 0, 0)); 
   /* Draw player on screen */
   dstrect.x = player->view->x;
   dstrect.y = player->view->y;
   dstrect.w = player->view->sprite->w;
   dstrect.h = player->view->sprite->h;
   SDL_BlitSurface(player->view->sprite, NULL, screen, &dstrect);
   /*SDL_UpdateRect(screen, 
                    player->view->x, player->view->y, 
                    player->view->sprite->w, player->view->sprite->h);*/
   dstrect.x = 0;
   dstrect.y = screen->h - player->view->sprite->h - PLAYER_Y_OFFSET;
   dstrect.w = screen->w;
   dstrect.h = player->view->sprite->h;
   SDL_UpdateRect(screen, dstrect.x, dstrect.y, dstrect.w, dstrect.h);

   return 1;
}

